#include "vecmath.h"
#include "animation.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "globals.h"

char playerShot1Data[] = { '*' };
char playerShot1ExplData[] = { '.' };
char enemyShot1Data[] = { 
'>', '0', '<', 
'*', '+', '*'};
char enemyShot1ExplData[] = { '.' };
	
char enemy1Data[] = {
//"  ___",
//"X{`m´}X", 
//" /\ /\"	
0,  0,  '_','_','_',0,0,
'X','{','`','m','\'','}','X',
0,  '/','\\',' ','/','\\',0,
//"  ___"
//"*{`m´}*" 
//"  /\ /\"
0,  0,  '_','_','_',0,0,
'*','{','`','m','\'','}','*',
0,  '/','\\',' ','/','\\',0,
//"  __"
//">{'-'}< "
//" /\ /\"
0,  0,  '_','_','_',0,0,
'>','{','\'','m','\'','}','<',
0,  '/','\\',' ','/','\\',0
};	

char player1Data[] = {
//"   I   ",
//")-(0)-(", 
//"  W W "	
0,  0,  0, 'I', 0, 0, 0,
')','-','(','0',')','-','(',
0,  0, 'W', 0, 'W', 0, 0,
//"  ___"
//"*{`m´}*" 
//"  /\ /\"
0,  0,  0, 'I', 0, 0, 0,
')','-','(','-',')','-','(',
0,  0, 'w', 0, 'w', 0, 0
};

char expl1Data[] = {	
0,  0,  0,  0,  0,  0,  0,
0,  0,  0,'*',  0,  0,  0,
0,  0,  0,  0,  0,  0,  0,

0,  0,'*',  0,'*',  0,  0,
0,'(','(','*',')',')',  0,
0,  0,'*',  0,'*',  0,  0,

  0,'*','*',  0,'*','*',  0,
  0,  0,'*','*','*',  0,  0,
'(','(','(','*',')',')',')',
  0,  0,'*','*','*',  0,  0,
  0,'*','*',  0,'*','*',  0,
 
'*',  0,  0,'*',  0,  0,'*',
  0,'*',  0,  0,  0,'*',  0,
'(','(',  0,  0,  0,')',')',
  0,'*',  0,  0,  0,'*',  0,
'*','*',  0,'*',  0,'*','*',

  0,'*',  0,  0,  0,'*',  0,
'*',  0,  0,  0,  0,  0,'*',
'(',  0,  0,  0,  0,  0,')',
'*',  0,  0,  0,  0,  0,'*',
  0,'*',  0,  0,  0,'*',  0,
};	

void createAnimFrames(Animation* anim, AnimFrame animFrames[], int w[], int h[], char* charData)
{
	float charWidth = simulatedWidth / screenWidth; // number of simulated pixels per screenWidth pixel (=1 terminal character)
	float charHeight = simulatedHeight / screenHeight;	
	char* strData = charData;
	int i;
	for(i=0; i<anim->nAnimations; i++) {
		anim->animFrame[i].str = strData;
		strData += w[i]*h[i];
		anim->animFrame[i].aabb.min.x = -w[i]/2.0 * charWidth;;
		anim->animFrame[i].aabb.min.y = -h[i]/2.0 * charHeight;
		anim->animFrame[i].aabb.max.x =  w[i]/2.0 * charWidth;
		anim->animFrame[i].aabb.max.y =  h[i]/2.0 * charHeight;
		anim->animFrame[i].nChars = w[i];
		anim->animFrame[i].nLines = h[i];			
	}
}

void drawAnimation(Animation* animation, int frame, Vec2f pos, char* frameBuffer)
{
	AABB2f box = Add(&animation->animFrame[frame].aabb, pos);
	// convert upper right aabb corner to screen position
	float charWidth = simulatedWidth / screenWidth; // number of simulated pixels per screenWidth pixel (=1 terminal character)
	float charHeight = simulatedHeight / screenHeight;		
	int left, top; // position of top left character in object
	left = (int)((box.min.x+0.5*charWidth) / charWidth); // round position to closest char position
	top = (int)((box.max.y+0.5*charHeight) / charHeight); // screen position in characters
	top--; // 1 character down from rounded top of bounding box.
	
	// Test if full object outside screen. If so, skip rendering
	if( top<0 || left  >= screenWidth || 
		top + 1 - animation->animFrame[frame].nLines >= screenHeight   || 
		left + animation->animFrame[frame].nChars <= 0)
			return;
	
	// print characters
	int y;
	int i=0;
	for(y=top; y>top - animation->animFrame[frame].nLines; y--) {
		int x;
		for(x=left; x<left + animation->animFrame[frame].nChars; x++) {
			if(x>=0 && x<screenWidth && y>=0 && y<screenHeight) {
				int ix = y*screenWidth + x;
				char c = animation->animFrame[frame].str[i];
				if(c != 0)
					frameBuffer[ix] = c;
			}
			i++;
		}
	}
}
void createEnemyShot1(Animation* shot)
{
	static AnimFrame AnimFrames[2];
	shot->nAnimations = 2;
	shot->cycleTime = 0.5f;
	shot->animTime = 0;
	shot->animFrame = AnimFrames;
	int w[] = {3.0f, 3.0f};	// with and height of object in #characters
	int h[] = {1.0f, 1.0f};	
	static int bFirstTime = 1;
	if(bFirstTime) {
		createAnimFrames(shot, AnimFrames, w, h, enemyShot1Data);
		bFirstTime = 0;
	}	
}

void createPlayerShot1(Animation* shot)
{
	static AnimFrame AnimFrames[1];
	shot->nAnimations = 1;
	shot->cycleTime = 1;
	shot->animTime = 0;
	shot->animFrame = AnimFrames;
	int w[] = {1.0f};	// with and height of object in #characters
	int h[] = {1.0f};	
	static int bFirstTime = 1;
	if(bFirstTime) {
		createAnimFrames(shot, AnimFrames, w, h, playerShot1Data);
		bFirstTime = 0;
	}
}
void createShotExpl1(Animation* shot)
{
	static AnimFrame AnimFrames[1];
	shot->nAnimations = 1;
	shot->cycleTime = 0.0001; // must be non-zero to avoid division by zero in main->getAABB()
	shot->animTime = 0;
	shot->animFrame = AnimFrames;
	int w[] = {1.0f};	// with and height of object in #characters
	int h[] = {1.0f};	
	static int bFirstTime = 1;
	if(bFirstTime) {
		createAnimFrames(shot, AnimFrames, w, h, playerShot1ExplData);
		bFirstTime = 0;
	}
}
void createEnemy1(Animation* enemy)
{
	static AnimFrame AnimFrames[3];
	enemy->nAnimations = 3;
	enemy->cycleTime = 3;
	enemy->animTime = 0;
	enemy->animFrame = AnimFrames;
	
	static int bFirstTime = 1;
	if(bFirstTime)
	{
		int w[] = {7, 7, 7};
		int h[] = {3, 3, 3};	
		createAnimFrames(enemy, AnimFrames, w, h, enemy1Data);
		bFirstTime = 0;
	}
}

void createPlayer1(Animation* player)
{
	static AnimFrame AnimFrames[2];
	player->nAnimations = 2;
	player->cycleTime = 2;
	player->animTime = 0;
	player->animFrame = AnimFrames;
	int w[] = {7, 7, 7};
	int h[] = {3, 3, 3};	
	static int bFirstTime = 1;
	if(bFirstTime)
	{
		createAnimFrames(player, AnimFrames, w, h, player1Data);
		bFirstTime = 0;
	}
}

void createExplosion1(Animation* expl)
{
	static AnimFrame animFrames[5];	
	expl->nAnimations = 5;
	expl->cycleTime = 1;
	expl->animTime = 0;
	expl->animFrame = animFrames;	
	int w[] = {7, 7, 7, 7, 7};
	int h[] = {3, 3, 5, 5, 5};		
	static int bFirstTime = 1;
	if(bFirstTime)
	{		
		createAnimFrames(expl, animFrames,  w, h, expl1Data);
		bFirstTime = 0;
	}
}


/*	
__|__  
  |
 \ /
 / \
 */
 