#include "enemies.h"
#include "player.h"
#include "globals.h"
#include <stdlib.h>
#include <math.h>

float timeNextAttack = 0.0f;

static int myRand(float min, float max) { return ( (min) + rand()/(RAND_MAX + 1.0) * ((max+1.0) - (min))); }
static float myRandf(float min, float max) { return ( min + rand()/(double)(RAND_MAX) * (max - min)); }

void insertMoreEnemies(float elapsedTime, EnemyArray* enemies)
{	
	if(elapsedTime < timeNextAttack)
		return;
	float avgTime = max(0.0f, 7.0f - 1.5f*elapsedTime/60.0f); 	
	float timeToNextAttack = myRand(avgTime/2.0f, avgTime);
	timeNextAttack = elapsedTime + timeToNextAttack;
	// loop enemies->enemy array and insert new enemies where there is room
	float avgNumEnemies = 4 + (2*elapsedTime/60.0f);
	int nNewEnemies = myRand(avgNumEnemies/2.0f, avgNumEnemies); 
	int i = 0, cntDown = nNewEnemies;
	
	// randomly select attack to appear from hemisphere, just outside the screen.
	//  Select a random value between (x=[0,simulatedWidth], y=[minH, maxH])
	
	float spanHeight = min(1, ((int)(elapsedTime/30)/16.0f)) * (simulatedHeight/2.0f - 5*charHeight);
	float height = myRand( max(1.5*charHeight, simulatedHeight/2.0f - spanHeight), simulatedHeight-1.5*charHeight); 
	// randomly select attack to appear on left or right side of screen
	int side = myRand(0,1);
	side = side ? 1 : -1;	// side = -1 or 1;

	float rotSpeed = myRand(1, min(5, elapsedTime/60));							
	for (i=0; i<enemies->l && cntDown > 0 ; i++) {
		if(enemies->enemy[i].o.status == DEAD) {
			cntDown--;
			createEnemy1(&(enemies->enemy[i].o.gfxData));
			createExplosion1(&(enemies->enemy[i].o.explosionData));
			enemies->enemy[i].o.health = 1;
			enemies->enemy[i].o.status = LIVE;
			enemies->enemy[i].o.properties = UPDATE_POS_FROM_VEL | SCREEN_BORDER_BOUNCE;
			enemies->enemy[i].o.velocity.x = -side*charWidth*(5+2*elapsedTime/60.0f); //simulatedWidth/15.0f;
			float span = 2*(int)(elapsedTime/60.0f);
			enemies->enemy[i].o.velocity.y = 0 + charHeight * (myRand(0, span)-span/2.0f); //simulatedHeight/10.0f;
			AABB2f box = getAABB(&enemies->enemy[i].o);
			float boxWidth = box.max.x - box.min.x;				
			enemies->enemy[i].o.pos.x = simulatedWidth/2.0f + side*(simulatedWidth/2.0f + boxWidth/2);
			enemies->enemy[i].o.pos.y = height; //simulatedHeight/2.0f;	
			enemies->enemy[i].o.prevPos = enemies->enemy[i].o.pos;
			
			// adjust for start of rotations in circle, such that the start in the circle is where the speed vector is tangent to the circle 
			Vec2f offset = enemies->enemy[i].o.velocity; 
			float l = 1.0f / (sqrt(offset.x*offset.x + offset.y*offset.y));
			offset.x *= l;
			offset.y *= l;
			offset.x = max(-1.0f, min(1.0f, offset.x)); // extra clamp in case of float errors
			offset.y = max(-1.0f, min(1.0f, offset.y));
			offset.x = asin(-offset.x); // f_x(t) = cos(t), f'_x(t) == v.x => t = asin(-v.x);
			offset.y = acos(offset.y); // f_y(t) = sin(t), f'_y(t) == v.y => t = acos(v.y);
			double t = (offset.y >= 0) ? offset.x : 2*PI - offset.x;
			enemies->enemy[i].offset[0] = enemies->enemy[i].offset[1] = enemies->enemy[i].offset[2] = t;
			enemies->enemy[i].rotDir[0] = enemies->enemy[i].rotDir[1] = enemies->enemy[i].rotDir[2] = 1;
			if(side == 1) {
				enemies->enemy[i].offset[0] = enemies->enemy[i].offset[1] = enemies->enemy[i].offset[2] = t + PI;
				enemies->enemy[i].rotDir[0] = enemies->enemy[i].rotDir[1] = enemies->enemy[i].rotDir[2] = -1;
			}
				
		
			// enemies->enemy[j].o.pos.xy += (r->x * (1.0f - 0.5f*sin(f->x*t + offset.x))) * sin(rotSpeed->x*t + offset.x);
			enemies->enemy[i].radius[0].x = 0; //3*charWidth; //simulatedWidth  / (float)screenWidth;
			enemies->enemy[i].radius[0].y = 0; //3*charHeight;
			enemies->enemy[i].radius[1].x = 10*charWidth; //simulatedWidth  / (float)screenWidth;
			enemies->enemy[i].radius[1].y = 5*charHeight; //simulatedHeight / (float)screenWidth;
			enemies->enemy[i].radius[2].x = 0; //simulatedWidth  / (float)screenWidth;
			enemies->enemy[i].radius[2].y = 0; //simulatedHeight / (float)screenWidth;
			
			enemies->enemy[i].rotSpeed[0].x = rotSpeed*2.0f;
			enemies->enemy[i].rotSpeed[0].y = rotSpeed*2.0f;
			enemies->enemy[i].rotSpeed[1].x = rotSpeed; //2+side;
			enemies->enemy[i].rotSpeed[1].y = rotSpeed; //2+side;
			enemies->enemy[i].rotSpeed[2].x = rotSpeed*0.5f;
			enemies->enemy[i].rotSpeed[2].y = rotSpeed*0.5f;
			
			enemies->enemy[i].radiusSpeed[0].x = 0;
			enemies->enemy[i].radiusSpeed[0].y = 0;
			enemies->enemy[i].radiusSpeed[1].x = 0;
			enemies->enemy[i].radiusSpeed[1].y = 0;
			enemies->enemy[i].radiusSpeed[2].x = 0;
			enemies->enemy[i].radiusSpeed[2].y = 0;
			enemies->enemy[i].radiusOffs[0] = enemies->enemy[i].radiusOffs[1] = enemies->enemy[i].radiusOffs[2] = PI/2; 

			enemies->enemy[i].o.aliveTime = 0.0f - (float)cntDown*boxWidth/abs(enemies->enemy[i].o.velocity.x);
			enemies->enemy[i].o.deathTime = DBL_MAX; 
		}
	}
}

void updateEnemyShots(EnemyShotArray* enemyShots, double deltaTime)
{
	int j;
	// Advance each shot by its speed vector * deltaTime;
	for(j=0; j<enemyShots->l; j++) {	
		if(enemyShots->shot[j].o.status != DEAD) {
			enemyShots->shot[j].o.prevPos = enemyShots->shot[j].o.pos;
			updateObject(&enemyShots->shot[j].o, deltaTime);
		}
	}		
}

void updateEnemies(EnemyArray* enemies, EnemyShotArray* enemyShots, Player* player, double deltaTime)
{
	int i=0, j;
	// Advance each enemy by its motion vector * deltaTime;
	for(j=0; j<enemies->l; j++) {
		if(enemies->enemy[j].o.status != DEAD) {
			enemies->enemy[j].o.prevPos = enemies->enemy[j].o.pos;			
			if(enemies->enemy[j].o.status == LIVE && enemies->enemy[j].o.aliveTime >= 0) {
				int k;
				for(k=0; k<3; k++) {
					Vec2f* r = &enemies->enemy[j].radius[k];
					Vec2f* rS = &enemies->enemy[j].radiusSpeed[k];
					double rO = enemies->enemy[j].radiusOffs[k];
					Vec2f* rotSpeed = &enemies->enemy[j].rotSpeed[k];
					double t = enemies->enemy[j].o.aliveTime + enemies->enemy[j].offset[k];
					//enemies->enemy[j].o.pos.x += (r->x * (1.0f - 0.5f*sin(f->x*t + offset.x))) * sin(rotSpeed->x*t + offset.x);
					//enemies->enemy[j].o.pos.y += (r->y * (1.0f - 0.5f*sin(f->x*t + offset.y))) * cos(rotSpeed->y*t + offset.y);

					float s = enemies->enemy[j].rotDir[k];
					float R = r->y * sin(t*rS->y + rO);
					enemies->enemy[j].o.pos.y += R * cos(t*s*rotSpeed->y)*deltaTime*rotSpeed->y; // dcos()/dt = -sin()
					R = r->x * sin(t*rS->x + rO);
					enemies->enemy[j].o.pos.x += -R * sin(t*s*rotSpeed->x)*deltaTime*rotSpeed->x;					
				}
			} 
			int ret = updateObject(&enemies->enemy[j].o, deltaTime);
			// if object bounced at screen border (i.e., velocity has flipped), then also flip the other motion parameters
			if( ret & (XBOUNCE|YBOUNCE) && enemies->enemy[j].o.status == LIVE) {
				int k;
				if(ret & YBOUNCE) {
					for(k=0; k<3; k++)
						enemies->enemy[j].radius[k].y = -enemies->enemy[j].radius[k].y;
				}
				if(ret & XBOUNCE) {
					for(k=0; k<3; k++)
						enemies->enemy[j].radius[k].x = -enemies->enemy[j].radius[k].x;
				}
			}
			
			
			// Insert enemy shots
			if(enemies->enemy[j].o.status == LIVE && enemies->enemy[j].o.aliveTime >= 0) {
				if(myRand(0,100) == 50) {
					// Find first free position and insert new shot
					for( ; i<enemyShots->l; i++) {
						if(enemyShots->shot[i].o.status == DEAD) {
							createEnemyShot1(&enemyShots->shot[i].o.gfxData);
							createShotExpl1(&enemyShots->shot[i].o.explosionData);

							Vec2f dir;
							dir.x = player->o.pos.x - enemies->enemy[j].o.pos.x;
							dir.y = player->o.pos.y - enemies->enemy[j].o.pos.y;
							float l = sqrt(dir.x*dir.x + dir.y*dir.y);
							dir.x /= l;
							dir.y /= l;
							enemyShots->shot[i].o.aliveTime = 0;
							enemyShots->shot[i].o.deathTime = DBL_MAX;
							enemyShots->shot[i].o.status = LIVE;
							enemyShots->shot[i].o.velocity.y = dir.y * simulatedWidth/3.0; // passes full screen in 3 seconds
							enemyShots->shot[i].o.velocity.x = dir.x * simulatedWidth/3.0;
							enemyShots->shot[i].o.health = 1;
							enemyShots->shot[i].o.properties = UPDATE_POS_FROM_VEL | DEAD_OFF_SCREEN;
							enemyShots->shot[i].o.pos = enemies->enemy[j].o.pos;
							enemyShots->shot[i].o.prevPos = enemyShots->shot[i].o.pos;

					/*		// Place shot outside bounding box of shooting enemy
							AABB2f box = getAABB(&enemyShots->shot[i].o);
							Vec2f shotHalf = { .v={(box.max.x-box.min.x)*0.5f, (box.max.y-box.min.y)*0.5f} };
							float lShot = sqrt(shotHalf.x*shotHalf.x + shotHalf.y*shotHalf.y);
							box = getAABB(&enemies->enemy[j].o);
							Vec2f enemyHalf = { .v={(box.max.x-box.min.x)*0.5f, (box.max.y-box.min.y)*0.5f} }; 
							float lEnem = sqrt(enemyHalf.x*enemyHalf.x + enemyHalf.y*enemyHalf.y);
							enemyShots->shot[i].o.pos.x += (lShot + lEnem) * (dir.x);	
							enemyShots->shot[i].o.pos.y += (lShot + lEnem) * (dir.y); */
							i++;
							break;
						}
					}
				}		
			}	
		}
	}		
}

void drawEnemies(EnemyArray* enemies)
{
	int j;
	for(j=0; j<enemies->l; j++) 
		drawObject(&enemies->enemy[j].o);
}

void drawEnemyShots(EnemyShotArray* enemyShots)
{
	int j;
	for(j=0; j<enemyShots->l; j++)
		drawObject(&enemyShots->shot[j].o);
}
