#include <stdio.h>
#include <time.h>
#ifndef WINDOWS
#include <sys/time.h>
#endif
#include <stdlib.h>
#include <conio.h>
#include <math.h>
//#ifdef WINDOWS
#include <Windows.h>
#include <Wincon.h>
//#endif

#include "vecmath.h"
#include "animation.h"
#include "enemies.h"
#include "player.h"
#include "rendertext.h"
#include "object.h"
#include "globals.h"

////////////////////////////////////////////////////////////////////////////////////////////
// TODO:
//	enemyshots, multiple player fire incl achieving/loosing it,  
// 	enemystrikes, respawn 30s back
//
////////////////////////////////////////////////////////////////////////////////////////////

int screenWidth = 80, screenHeight = 40;
int simulatedWidth = 1920, simulatedHeight = 1200;
float charWidth; // number of simulated pixels per screenWidth pixel (=1 terminal character)
float charHeight; 	
char *frameBuffer = NULL;
Vec2f playerSpeed;
float frameTime = 25; //ms

Player player;
EnemyArray enemies = {NULL, 0};
PlayerShotArray playerShots = {NULL, 0};
EnemyShotArray enemyShots = {NULL, 0};
TextObjectArray textObjects = {NULL, 0};

void initLevel(double elapsedTime)
{
	// Clear all arrays
	if(enemies.enemy != NULL)
		free(enemies.enemy);
	enemies.l = 0;

	if(enemyShots.shot != NULL)
		free(enemyShots.shot);
	enemyShots.l = 0;

	if(playerShots.shot != NULL)
		free(playerShots.shot);
	playerShots.l = 0;
	
	if(textObjects.letters != NULL)
		free(textObjects.letters);
	textObjects.l = 0;
	
	// Initiate all arrays with free objects
	int numObjects = 50;
	enemies.enemy = (Enemy*)malloc(numObjects * sizeof(Enemy));
	int i;
	for(i = 0; i<numObjects; i++)
		enemies.enemy[i].o.status = DEAD;
	enemies.l = numObjects;
		
	numObjects = 100;
	enemyShots.shot = (EnemyShot*)malloc(numObjects * sizeof(EnemyShot));
	for(i = 0; i<numObjects; i++)
		enemyShots.shot[i].o.status = DEAD;
	enemyShots.l = numObjects;

	textObjects.letters = (TextObject*)malloc(numObjects * sizeof(TextObject));
	for(i = 0; i<numObjects; i++)
		textObjects.letters[i].o.status = DEAD;	
	textObjects.l = numObjects;

	numObjects = 1000;
	playerShots.shot = (PlayerShot*)malloc(numObjects * sizeof(PlayerShot));
	for(i = 0; i<numObjects; i++)
		playerShots.shot[i].o.status = DEAD;	
	playerShots.l = numObjects;
	
	// Init player
	createPlayer1(&(player.o.gfxData));
	createExplosion1(&(player.o.explosionData));
	player.o.health = 1;
	player.o.aliveTime = 0.0;
	player.o.deathTime = DBL_MAX;
	player.o.status = LIVE;
	player.o.pos.x = simulatedWidth / 2.0f;
	player.o.pos.y = (player.o.gfxData.animFrame[0].aabb.max.y - player.o.gfxData.animFrame[0].aabb.min.y);
	player.o.prevPos = player.o.pos;
	player.o.velocity.x = simulatedWidth  / (0.5f+(min(1.5f, 1/((elapsedTime)*1.5/60.0)))); // seconds to strafe full screen. Starts at 2s and asymptotically goes to 0.5s
	player.o.velocity.y = simulatedHeight / (0.5f+(min(1.5f, 1/((elapsedTime)*1.5/60.0))));
	playerSpeed = player.o.velocity;
	player.nHits = 0;
	player.timePreviousShot = 0;
	player.fireDeltaTime = charHeight / (playerSpeed.y); // shot speed = simulatedHeight/3	
	player.o.properties = CLAMP_TO_SCREEN | LETHAL_ON_EXPLODING; // SCREEN_BORDER_BOUNCE  | ;
}

void collisionDetection()
{
	// Test all player shots against all enemies
	collDetectObjects(&playerShots.shot[0].o, sizeof(PlayerShot), playerShots.l, &enemies.enemy[0].o, sizeof(Enemy), enemies.l);

	// Test all enemy shots against the player
	player.nHits += collDetectObjects(&player.o, sizeof(Player), 1, &enemyShots.shot[0].o, sizeof(EnemyShot), enemyShots.l);	
	
	// Test all enemies against the player
	player.nHits += collDetectObjects(&player.o, sizeof(Player), 1, &enemies.enemy[0].o, sizeof(Enemy), enemies.l);	
}


void clearFrameBuffer(char* buf)
{
	int i;
	for(i=0; i<screenWidth * screenHeight; i++)
		buf[i] = 0;
}
void clrscr()
{
	// Windows does not have the clrscr()-command in conio.h, so we add it manually
	// For us, it is enough to just set the cursor position to the top of screen
	COORD cursor = {0,0};
	SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), cursor);
}

int panel_w = 20, panel_h = 0;
void initWindow()
{
	COORD coord; 
    coord.X = (screenWidth+1) + panel_w; // for some reason, make console's buffer one element larger on MS Windows.
    coord.Y = (screenHeight) + panel_h;  // Probably, due to newline '\n', when drawing each line of the frameBuffer
	
    SMALL_RECT Rect; 
    Rect.Top = 0; 
    Rect.Left = 0; 
    Rect.Bottom = (screenHeight-1) + panel_h; 
    Rect.Right = (screenWidth-1) + panel_w; 
	
    HANDLE Handle = GetStdHandle(STD_OUTPUT_HANDLE);      // Get Handle 
    BOOL ret = SetConsoleScreenBufferSize(Handle, coord);            // Set Buffer Size 
    if(ret)
		ret=SetConsoleWindowInfo(Handle, TRUE, &Rect);            // Set Window Size
}
void drawFrameBuffer(char* buf, int level, int levelLives, double gameTime)
{
	clrscr(); // clears console screen and places cursor at top
	int i,j;
	char line[screenWidth+1 + panel_w];
	// print each line of the frame buffer
	// Draw with origin at lower left corner
	for(i=screenHeight-1; i>=0; i--) {
		for(j=0; j<screenWidth; j++)
		{
			char c = buf[i*screenWidth + j];
			line[j] = (c == 0) ? ' ' : c;
		}
		line[j++] = '|';
		line[j] = 0; // NULL-terminate string
		if(i==screenHeight-2)
			sprintf(&line[j++],"  Level %i", level);
		if(i==screenHeight-4)
			sprintf(&line[j++],"  Health: %i", levelLives);
		if(i==screenHeight-6)
			sprintf(&line[j++],"  Time: %05d", (int)gameTime);
		if(i==0)
			printf("%s", line); // don't print newline if last line, because that will scroll screen one line
		else
			printf("%s\n", line);
	}	
}

int main()
{	
	charWidth = simulatedWidth / screenWidth; // number of simulated pixels per screenWidth pixel (=1 terminal character)
	charHeight = simulatedHeight / screenHeight;	

	BOOL bGameOver = FALSE;
	frameBuffer = malloc(screenWidth * screenHeight * sizeof(char));
	initWindow();

	int level = 1;
	double gameTime = (level-1)*60, levelStartTime = 0;
	struct timeval gameStartTime;
	// set a unique random seed - to get a different game each play
	gettimeofday(&gameStartTime, NULL);
	double nowTime = (gameStartTime.tv_sec);
	int rseed = fmod(nowTime, 1000000.0f); 
	srand(rseed);	
	
	do
	{
		// set up a new level
		levelStartTime = gameTime; 
		srand(rseed+(int)levelStartTime);	// make this level same, for each replay during this game	
		initLevel(levelStartTime);
		
		char str[50];
		sprintf(str, "Level %i", level);
		printCenteredTxt(&textObjects, str);
		
		struct timeval frameStart, frameEnd;	
		gettimeofday(&frameStart, NULL);

		int bLevelFinished = FALSE, levelLives = 3;
	    double levelTime = 0, respawnTime = 0;
		do // play this level
		{
			gettimeofday(&frameEnd, NULL); // First frame, this will be ~zero, which is avoids updating positions before 1st display
			double deltaTime = (frameEnd.tv_sec * 1000000 + frameEnd.tv_usec) - (frameStart.tv_sec * 1000000 + frameStart.tv_usec);
			deltaTime /= 1000000.0f; // in seconds
			double gameSpeedup = 1.0;
			deltaTime *= gameSpeedup;
			frameStart = frameEnd;
			gameTime += deltaTime;
			levelTime += deltaTime;
			
			struct timeval loopStartTime;		// measure how long time the game logic takes and adjust sleep time to that
			gettimeofday(&loopStartTime, NULL);
			
			insertMoreEnemies(gameTime, &enemies);
			
			int prevPlayerStatus = player.o.status;
			collisionDetection();

			updatePlayer(&player, deltaTime);
			updatePlayerShots(&playerShots, &player, deltaTime, gameTime);
			updateEnemyShots(&enemyShots, deltaTime);
			updateEnemies(&enemies, &enemyShots, &player, deltaTime);
			updateTextObjects(&textObjects, deltaTime);
			
			clearFrameBuffer(frameBuffer);
			drawPlayer(&player);
			drawPlayerShots(&playerShots);
			drawEnemyShots(&enemyShots);
			drawEnemies(&enemies);
			drawTextObjects(&textObjects);

			drawFrameBuffer(frameBuffer, level, levelLives, gameTime);
			
			struct timeval loopEndTime;		// measure how long time the game logic takes and adjust sleep time to that
			gettimeofday(&loopEndTime, NULL);
			double loopTime = (loopEndTime.tv_sec * 1000000 + loopEndTime.tv_usec) - (loopStartTime.tv_sec * 1000000 + loopStartTime.tv_usec);
			loopTime /= 1000.0f; // in milliseconds

			//int sleepTime = max(0,frameTime-loopTime);
			int sleepTime = max(0, min(frameTime, frameTime - (deltaTime*1000.0-frameTime)));
			Sleep(sleepTime); // Most monitor refresh rates can be evenly divided by four.
			
			
			// Cheat logic
			if(GetAsyncKeyState(0x4C) &0x8000) { // jump to next level
				gameTime += 60-levelTime;
				levelTime = 60;
			}
			
			
			// Game level logic

			if(prevPlayerStatus == LIVE && player.o.status != LIVE)
				levelLives--;
			if(player.o.status == DEAD && player.o.health <= 0) {
				player.o.health = 1; // signal not to enter here next frames
				respawnTime = gameTime + 5.0; // wait 5 seconds to respawn
			}
			if(respawnTime > gameTime) {
				respawnTime = 0;
				player.o.health = 1;
				player.o.status = LIVE;
				if(levelLives <= 0)
					bLevelFinished = TRUE; // restart the same level
			}
			if(levelTime > 60)
				bLevelFinished = TRUE;
			
		} while(!bLevelFinished);
		if(levelLives <= 0) {
			gameTime = levelStartTime;  // Adjusting gameTime messes with timeToNextAttack,
			timeNextAttack = 0;			// so, reset it.
		}
		else
			level++;
			
	} while(!bGameOver);
	
	return 0;
}

