#include "object.h"
#include <stdlib.h>
#include <math.h>
#include "globals.h"


AABB2f getAABB(Object* o)
{
	AABB2f box;	
	if(o->status == LIVE) {
		float timePerAnimFrame = o->gfxData.cycleTime / o->gfxData.nAnimations;
		float anim = o->gfxData.animTime / timePerAnimFrame;
		anim = max(0.0f, min(anim, o->gfxData.nAnimations-1));
		box = o->gfxData.animFrame[(int)anim].aabb;
	}
	else if(o->status == EXPLODING) {
		float timePerAnimFrame = o->explosionData.cycleTime / o->explosionData.nAnimations;
		float anim = o->explosionData.animTime / timePerAnimFrame;
		anim = max(0.0f, min(anim, o->explosionData.nAnimations-1));
		box = o->explosionData.animFrame[(int)anim].aabb;			
	}
	box = Add(&box, o->pos);
	return box;
}

void drawObject(Object* o)
{
	if(o->status == LIVE && o->aliveTime >= 0) {
		float timePerAnimFrame = o->gfxData.cycleTime / o->gfxData.nAnimations;
		float anim = o->gfxData.animTime / timePerAnimFrame;
		anim = max(0.0f, min(anim, o->gfxData.nAnimations-1));
		drawAnimation(&o->gfxData, (int)anim, o->pos, frameBuffer);
	}
	else if(o->status == EXPLODING) {
		float timePerAnimFrame = o->explosionData.cycleTime / o->explosionData.nAnimations;
		float anim = o->explosionData.animTime / timePerAnimFrame;
		anim = max(0.0f, min(anim, o->explosionData.nAnimations-1));
		drawAnimation(&o->explosionData, (int)anim, o->pos, frameBuffer);
	}
}

// Since C lacks templates...
// Ugly way to avoid writing this same code for each Array vs Array, where each array element contains an instance "object".
int collDetectObjects(Object* o1, int structSize1, int nElems1, Object* _o2, int structSize2, int nElems2)
{
	int i,j, numHits=0;
	Object* o2;

	// Test all objects  against all enemies
	for(j=0; j<nElems1; j++) {
		if(o1->status != DEAD && o1->aliveTime >= 0) { // Dead object can't hurt other object
			o2 = _o2;
			for( i=0; i<nElems2; i++) {
				if(o2->status != DEAD && o2->aliveTime >= 0) { // Dead object can't hurt other object
					if(o1->status != EXPLODING || o2->status != EXPLODING) // Two exploding objects can't hurt each other
					{
						// test overlap for boundingboxes
						AABB2f box1, box2;
						box1 =  getAABB(o1);
						box2 =  getAABB(o2);
						if(colliding(&box1, &box2)) {
							// Check who kills who. Each object is either live or exploding
							int killObj1 = (o2->status == LIVE || o2->properties & LETHAL_ON_EXPLODING);
							int killObj2 = (o1->status == LIVE || o1->properties & LETHAL_ON_EXPLODING);
							if( killObj1 && o1->status == LIVE && (--o1->health) <= 0 ) { // Note that decrement of health is only done if status==LIVE
								o1->status = EXPLODING;
								o1->explosionData.animTime = 0.0f;
							}								
							if( killObj2 && o2->status == LIVE && (--o2->health) <= 0 ) {
								o2->status = EXPLODING;	
								o2->explosionData.animTime = 0.0f;
							}
						}
					}
				}
				(*(void**)&o2) += structSize2; // increase object pointer by structSize
			}
		}
		(*(void**)&o1) += structSize1;
	}	
	return numHits;
}

int updateObject(Object *o, double deltaTime)
{
	o->aliveTime += deltaTime;
	if(o->aliveTime < 0)
		return 0;
		
	if(o->deathTime <= o->aliveTime)
		o->status = EXPLODING;
		
	if(o->status == DEAD)
		return 0; // Code below asumes object is LIVE or EXPLODING
		
	// Update animation
	if(o->status == EXPLODING)
	{
		o->explosionData.animTime += deltaTime;
		if(o->explosionData.animTime > o->explosionData.cycleTime) {
			o->explosionData.animTime = 0.0f;
			o->status = DEAD;
		}
	}
	if(o->status == LIVE) 
	{
		o->gfxData.animTime += deltaTime;
		o->gfxData.animTime = fmod(o->gfxData.animTime, o->gfxData.cycleTime);			
	}
	
	// Update position
	if( o->status == LIVE && (o->properties & UPDATE_POS_FROM_VEL) ) {
		o->pos.x += o->velocity.x * deltaTime;
		o->pos.y += o->velocity.y * deltaTime;
	}
	
	// Get current bounding box of object
	AABB2f box = getAABB(o);	
	
	// Kill object if it is fully outside the screen
	if( o->properties & DEAD_OFF_SCREEN ) {
		if(box.min.x-charWidth > simulatedWidth || box.max.x+charWidth < 0 || box.min.y-charHeight > simulatedHeight || box.max.y+charHeight < 0)
			o->status = DEAD;
	}
	
	// Clamp position to be inside screen
	float halfBoxWidth = (box.max.x - box.min.x)*0.5f;
	float halfBoxHeight = (box.max.y - box.min.y)*0.5f;	
	if( o->status == LIVE && (o->properties & CLAMP_TO_SCREEN) ) {

		o->pos.x = min(o->pos.x,  (simulatedWidth)-halfBoxWidth);
		o->pos.x = max(o->pos.x, halfBoxWidth);		
		o->pos.y = min(o->pos.y,  (simulatedHeight-1)-halfBoxHeight);
		o->pos.y = max(o->pos.y, halfBoxHeight);					
	}
	
	// Bounce object against screen borders. I.e., reflect corresponding velocity
	int retval = 0;
	if( o->status == LIVE && (o->properties & SCREEN_BORDER_BOUNCE) ) {
		// compute move direction. Don't bounce if object is moving from outside screen into screen
		float moveX = o->velocity.x; //o->pos.x - o->prevPos.x;
		float moveY = o->velocity.y; //o->pos.y - o->prevPos.y;	
		// Check bounce at left or right
		float diffRight = (o->pos.x + halfBoxWidth) - (simulatedWidth-1); // # pixels out to the right
		float diffLeft = (o->pos.x - halfBoxWidth); // #pixels out to the left (negative)
		if(diffRight>0 && moveX > 0) {
			o->pos.x -= 2*diffRight;
			o->velocity.x = -o->velocity.x;
			retval |= XBOUNCE;
		}
		if(diffLeft<0 && moveX < 0) {
			o->pos.x -= 2*diffLeft;
			o->velocity.x = -o->velocity.x;
			retval |= XBOUNCE;
		}
		// Check bounce at top or bottom
		float diffTop = (o->pos.y + halfBoxHeight) - (simulatedHeight-1); // # pixels out to the top
		float diffBottom = (o->pos.y - halfBoxHeight); // #pixels out to the bottom (negative)	
		if(diffTop>0 && moveY > 0) {
			o->pos.y -= 2*diffTop;
			o->velocity.y = -o->velocity.y;
			retval |= YBOUNCE;
		}
		if(diffBottom<0 && moveY < 0) {
			o->pos.y -= 2*diffBottom;
			o->velocity.y = -o->velocity.y;
			retval |= YBOUNCE;
		}		
	}	
	return retval;
}