#include "player.h"
//#ifdef WINDOWS
#include <Windows.h>
//#endif
#include "globals.h"


void updatePlayer(Player* player, double deltaTime) // deltaTime is fractions of a second since last frame
{
	player->o.prevPos = player->o.pos;
	/////////////////////////
	// Read keyboard
	/////////////////////////
	
	// this works on most operating systems, but you cannot press more than one key at a time
	// int c = 0;
   	// if(kbhit()) 
	//	 c = getch();
	
	// Microsoft Windows solution
	if(player->o.status == LIVE)
	{
		if(GetAsyncKeyState(VK_LEFT) &0x8000) 
			player->o.pos.x -= player->o.velocity.x * deltaTime;
		if(GetAsyncKeyState(VK_RIGHT) &0x8000)
			player->o.pos.x += player->o.velocity.x * deltaTime;
		if(GetAsyncKeyState(VK_UP) &0x8000)
			player->o.pos.y += player->o.velocity.y * deltaTime;
		if(GetAsyncKeyState(VK_DOWN) &0x8000)
			player->o.pos.y -= player->o.velocity.y * deltaTime;	
	}
	updateObject(&player->o, deltaTime);	
}

void updatePlayerShots(PlayerShotArray* playerShots, Player* player, double deltaTime, double currentTime)
{
	int i;	
	// Move each shot upwards one step on screen
	for(i=0; i<playerShots->l; i++) {
		if(playerShots->shot[i].o.status != DEAD) {
			playerShots->shot[i].o.prevPos = playerShots->shot[i].o.pos;
			updateObject(&playerShots->shot[i].o, deltaTime);
		}
	}
	
	// Add new shot if left mouse button or space is pressed
	if( player->o.status == LIVE && 
	  ( player->timePreviousShot + player->fireDeltaTime <= currentTime) ) // can there be a new shot up to this frame?
	{
		if( (GetAsyncKeyState(VK_LBUTTON) & 0x8000 ) || ( GetAsyncKeyState(VK_SPACE) & 0x8000 ) ) 
		{
			double adjustTime = 0;
			double t;
			// Antag att det skall införas skott från timePrevShots till nu.
			// loopa från t=timePreviousShot + i*fireDeltaTime tills > currentTime. 
			// 	 adjustTime = currentTime-t;
			
			// If the player fires, then the time for the next shot is t = timePrevShot + player->fireDeltaTime
			// If t < currentTime - deltaTime, then player did not fire during last frame 
			//   so set adjustTime=0, timePrevShot=currentTime, and t=currentTime.
			t = player->timePreviousShot + player->fireDeltaTime; // time of next shot
			if(t < currentTime - 1.4*deltaTime) { // player did not fire last frame
				player->timePreviousShot = currentTime - player->fireDeltaTime; // but signal that player did, so we insert one shot below
			}
				
			// Insert shots from time "timePrevShots" up to currentTime
			// loop from t = timePrevShots + i*fireDeltaTime, until > currentTime
			for(t=player->timePreviousShot+player->fireDeltaTime; t<=currentTime; t+=player->fireDeltaTime){
				adjustTime = currentTime - t;

				// Insert a shot
				
				AABB2f box = getAABB(&player->o);
				float playerHalfH = (box.max.y-box.min.y)*0.5f;	
				int numNewShots = min(9, 1 + currentTime/60);
				Vec2f *offsets, *dir;
				Vec2f offs1[] = {                                                           {.v={0,0} } };
				Vec2f offs2[] = {                                             {.v={-1, 0}},             {.v={1,0}} };
				Vec2f offs3[] = {                               {.v={-3,-1}},               {.v={0,0}},             {.v={3,-1}} };
				Vec2f offs4[] = { {.v={-4, -2}},                              {.v={-1, 0}},             {.v={1,0}}, 			 			   {.v={4, -2}}};
				Vec2f offs5[] = { {.v={-4, -2}}, {.v={-4, -1}},				                {.v={0,0}},            				 {.v={4, -1}}, {.v={4, -2}}};
				Vec2f offs6[] = { {.v={-4, -2}}, {.v={-4, -1}},	              {.v={-1, 0}},             {.v={1,0}}, 			 {.v={4, -1}}, {.v={4, -2}}};;
				Vec2f offs7[] = { {.v={-4, -2}}, {.v={-4, -1}}, {.v={-3,-1}}, 				{.v={0,0}}, 			{.v={3,-1}}, {.v={4, -1}}, {.v={4, -2}}};	
				Vec2f offs8[] = { {.v={-4, -2}}, {.v={-4, -1}}, {.v={-3,-1}}, {.v={-1, 0}},             {.v={1,0}}, {.v={3,-1}}, {.v={4, -1}}, {.v={4, -2}} };	
				Vec2f offs9[] = { {.v={-4, -2}}, {.v={-4, -1}}, {.v={-3,-1}}, {.v={-1, 0}}, {.v={0,0}}, {.v={1,0}}, {.v={3,-1}}, {.v={4, -1}}, {.v={4, -2}} };			
				Vec2f dir1[]  = {                                                           {.v={0,1} } };
				Vec2f dir2[]  = {                                             {.v={ 0, 1}},             {.v={0,1}} };
				Vec2f dir3[]  = {                               {.v={ 0, 1}},               {.v={0,1}},             {.v={0, 1}} };
				Vec2f dir4[]  = { {.v={-1,  0}},                              {.v={ 0, 1}},             {.v={0,1}}, 			 			   {.v={1,  0}}};
				Vec2f dir5[]  = { {.v={-1,  0}}, {.v={-1,  1}},				                {.v={0,1}},            				 {.v={1,  1}}, {.v={1,  0}}};
				Vec2f dir6[]  = { {.v={-1,  0}}, {.v={-1,  1}},	              {.v={ 0, 1}},             {.v={0,1}}, 			 {.v={1,  1}}, {.v={1,  0}}};;
				Vec2f dir7[]  = { {.v={-1,  0}}, {.v={-1,  1}}, {.v={ 0, 1}}, 				{.v={0,1}}, 			{.v={0, 1}}, {.v={1,  1}}, {.v={1,  0}}};	
				Vec2f dir8[]  = { {.v={-1,  0}}, {.v={-1,  1}}, {.v={ 0, 1}}, {.v={ 0, 1}},             {.v={0,1}}, {.v={0, 1}}, {.v={1,  1}}, {.v={1,  0}} };	
				Vec2f dir9[]  = { {.v={-1,  0}}, {.v={-1,  1}}, {.v={ 0, 1}}, {.v={ 0, 1}}, {.v={0,1}}, {.v={0,1}}, {.v={0, 1}}, {.v={1,  1}}, {.v={1,  0}} };					
				
				Vec2f* offs[] = {offs1, offs1, offs2, offs3, offs4, offs5, offs6, offs7, offs8, offs9}; 
				offsets = offs[numNewShots];			
				Vec2f* dirs[] = {dir1, dir1, dir2, dir3, dir4, dir5, dir6, dir7, dir8, dir9}; 
				dir = dirs[numNewShots];
				int j;
				// Find first free position and insert new shot
				for(i=0, j=0; i<playerShots->l && j<numNewShots; i++) {
					if(playerShots->shot[i].o.status == DEAD) {
						createPlayerShot1(&playerShots->shot[i].o.gfxData);
						createShotExpl1(&playerShots->shot[i].o.explosionData);

						playerShots->shot[i].o.aliveTime = 0;
						playerShots->shot[i].o.deathTime = DBL_MAX;
						playerShots->shot[i].o.status = LIVE;
						playerShots->shot[i].o.velocity.y = dir[j].y * playerSpeed.y*1; // passes full screen in 3 seconds
						playerShots->shot[i].o.velocity.x = dir[j].x * playerSpeed.x*1;
						playerShots->shot[i].o.health = 1;
						playerShots->shot[i].o.properties = UPDATE_POS_FROM_VEL | DEAD_OFF_SCREEN;
						
						playerShots->shot[i].o.pos.x = player->o.pos.x + offsets[j].x*charWidth;
						playerShots->shot[i].o.prevPos = playerShots->shot[i].o.pos;
						
						AABB2f box = getAABB(&playerShots->shot[i].o);
						float shotHalfH = (box.max.y-box.min.y)*0.5f;
						playerShots->shot[i].o.pos.y = player->o.pos.y + playerHalfH + shotHalfH + offsets[j].y*charHeight;	
						
						// Adjust position due to framerate discretation
						playerShots->shot[i].o.pos.x += playerShots->shot[i].o.velocity.x * adjustTime;
						playerShots->shot[i].o.pos.y += playerShots->shot[i].o.velocity.y * adjustTime;
						j++;
					}
				}
			}
			player->timePreviousShot = t-player->fireDeltaTime;
		}
	}
}

void drawPlayer(Player* player)
{
	drawObject(&player->o);
}

void drawPlayerShots(PlayerShotArray* playerShots)
{
	int j;
	for(j=0; j<playerShots->l; j++) 
		drawObject(&playerShots->shot[j].o);
}