#include "vecmath.h"
#include "animation.h"
#include "rendertext.h"
#include "globals.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

char txtL[] = {
'*', ' ',   0,   0, 
'*', ' ',   0,   0,
'*', ' ',   0,   0,
'*', ' ', ' ', ' ',  
'*', '*', '*', '*'
};

char txtE[] = {
'*', '*', '*', '*', 
'*', ' ',   0,   0,
'*', '*', '*', ' ',
'*', ' ', ' ', ' ',  
'*', '*', '*', '*'
};

char txtV[] = {
'*', ' ',   0, '*', 
'*', ' ',   0, '*',
'*', ' ', ' ', '*',
'*', ' ', '*', ' ',  
  0, '*', ' ',   0
};	
char txtSpace[] = {
  0,   0,   0,   0, 
  0,   0,   0,   0,
  0,   0,   0,   0,
  0,   0,   0,   0,  
  0,   0,   0,   0
};	
char txt0[] = {
' ', '*', '*', ' ', 
'*', ' ', ' ', '*',
'*', ' ', ' ', '*',
'*', ' ', ' ', '*',  
  0, '*', '*',   0
};	
char txt1[] = {
  0, '*', '*', '*', 
  0, ' ', '*', ' ',
  0, ' ', '*', ' ',
  0, ' ', '*', ' ',  
  0, '*', '*', '*'
};
char txt2[] = {
'*', '*', '*', ' ', 
  0, ' ', ' ', '*',
  0, ' ', '*', ' ',
  0, '*', ' ', ' ',  
'*', '*', '*', '*'
};
char txt3[] = {
'*', '*', '*', '*', 
  0, ' ', ' ', '*',
  0, '*', '*', '*',
  0, ' ', ' ', '*',  
'*', '*', '*', '*'
};
char txt4[] = {
'*', ' ', ' ', '*', 
'*', ' ', ' ', '*',
'*', '*', '*', '*',
  0, ' ', ' ', '*',  
  0,   0, ' ', '*'
};
char txt5[] = {
'*', '*', '*', '*', 
'*', ' ', ' ', ' ',
'*', '*', '*', '*',
' ', ' ', ' ', '*',  
'*', '*', '*', '*'
};
char txt6[] = {
' ', '*', '*', '*', 
'*', ' ', ' ', ' ',
'*', '*', '*', ' ',
'*', ' ', ' ', '*',  
' ', '*', '*', ' '
};
char txt7[] = {
'*', '*', '*', '*', 
' ', ' ', ' ', '*',
' ', ' ', '*', ' ',
' ', '*', ' ', ' ',  
'*', ' ', ' ', ' '
};
char txt8[] = {
' ', '*', '*', ' ', 
'*', ' ', ' ', '*',
' ', '*', '*', ' ',
'*', ' ', ' ', '*',  
' ', '*', '*', ' '
};
char txt9[] = {
' ', '*', '*', ' ', 
'*', ' ', ' ', '*',
' ', '*', '*', '*',
' ', ' ', ' ', '*',  
'*', '*', '*', ' '
};
char *numTbl[] = {txt0, txt1, txt2, txt3, txt4, txt5, txt6, txt7, txt8, txt9};
char *txtTbl[] = {0,0,0,0,txtE,0,0,0,0,0,0,txtL,0,0,0,0,0,0,0,0,0,txtV,0,0,0,0,0,0,0,txtSpace};

void createLetter(Animation* letter, char c)
{
	static AnimFrame charFrames[(sizeof(numTbl) + sizeof(txtTbl))/sizeof(char*) + 1];
	int nChars = (sizeof(numTbl) + sizeof(txtTbl))/sizeof(char*) + 1;
	
	// Fill all charFrames with right data
	static int bFirstTime = TRUE;
	if(bFirstTime) {
		int i;
		int w = 4;
		int h = 5;
		for(i=0; i<nChars; i++) {
			if(i<10)
				charFrames[i].str = numTbl[i];
			else 
				charFrames[i].str = txtTbl[i-10];
			charFrames[i].aabb.min.x = -w/2.0 * charWidth;;
			charFrames[i].aabb.min.y = -h/2.0 * charHeight;
			charFrames[i].aabb.max.x =  w/2.0 * charWidth;
			charFrames[i].aabb.max.y =  h/2.0 * charHeight;
			charFrames[i].nChars = w;
			charFrames[i].nLines = h;			
		}
		bFirstTime = FALSE;
	}
	
	// Make letter an AnimFrame that points to the right character data
	letter->nAnimations = 1;
	letter->cycleTime = 100;
	letter->animTime = 0;
	
	int l;
	if(c >= 48 && c <= 57) {
		l = c - 48;
		letter->animFrame = &charFrames[l];
	}
	else if(c>=65 && c <= 90) {
		l = c-65 + 10;
		letter->animFrame = &charFrames[l];
	}
	else if(c>=97 && c<=122) {
		l = c-97 + 10;
		letter->animFrame = &charFrames[l];
	
	}
	else if(c==32) {
		l = 29+10;
		letter->animFrame = &charFrames[l];
	}
	else // just as backup
		letter->animFrame = &charFrames[0];
}

void printCenteredTxt(TextObjectArray* textObjects, char* str)
{
	// Create an Animation for each charachter in string, and add the Animation to textObjects
	int l = strlen(str);
	int i, j;
	for(i=0, j=0; i<textObjects->l && j<l; i++)
	{
		if(textObjects->letters[i].o.status == DEAD) {
			createLetter(&textObjects->letters[i].o.gfxData, str[j]);
			createExplosion1(&textObjects->letters[i].o.explosionData);
			textObjects->letters[i].o.health = 1;
			textObjects->letters[i].o.aliveTime = 0.0;
			textObjects->letters[i].o.deathTime = 3.0;
			textObjects->letters[i].o.status = LIVE;
			float height = (textObjects->letters[i].o.gfxData.animFrame[0].aabb.max.y - textObjects->letters[i].o.gfxData.animFrame[0].aabb.min.y);
			float width = (textObjects->letters[i].o.gfxData.animFrame[0].aabb.max.x - textObjects->letters[i].o.gfxData.animFrame[0].aabb.min.x);
			textObjects->letters[i].o.pos.x = simulatedWidth / 2.0f + (j-l*0.5)*width*1.5f;
			textObjects->letters[i].o.pos.y = simulatedHeight/2.0f + height/2.0f;
			textObjects->letters[i].o.prevPos = textObjects->letters[i].o.pos;
			textObjects->letters[i].o.velocity.x = charWidth; // 2 seconds to strafe full screen
			textObjects->letters[i].o.velocity.y = 0;	
			textObjects->letters[i].o.properties = 0;
			j++;
		}
	}
}

void drawTextObjects(TextObjectArray* textObjects)
{
	int j;
	for(j=0; j<textObjects->l; j++) 
		drawObject(&textObjects->letters[j].o);
}

void updateTextObjects(TextObjectArray* textObjects, double deltaTime)
{
	int i;	
	for(i=0; i<textObjects->l; i++) {
		if(textObjects->letters[i].o.status != DEAD) {
			textObjects->letters[i].o.prevPos = textObjects->letters[i].o.pos;
			updateObject(&textObjects->letters[i].o, deltaTime);
		}
	}
}
